"""Top-level constant definition for ati18n."""

OUTPUT_HEAD = {'tools': 'ati18n', 'version': '0.1.0', 'app_type': 'java'}
ERROR_1001 = {
    'no': '1001',
    'level': '错误',
    'scope': '整体翻译',
    'name': '翻译条目数不匹配',
    'data': '检查的数据文件和相关信息',
    "comment": ''}
ERROR_1002 = {
    'no': '1002',
    'level': '错误',
    'scope': '整体翻译',
    'name': '文件名指定的语言种类和文件内容不匹配',
    'data': '检查的数据文件和相关信息',
    "comment": ''}
ERROR_2001 = {
    'no': '2001',
    'level': '警告/错误',
    'scope': '单个翻译',
    'name': '翻译项和指定的语言类型不匹配',
    'data': '检查的数据文件和相关信息',
    "comment": ''}



__author__ = 'painterg'
__email__ = '22396997@qq.com'

import py3langid as langid
import datetime
import pytz
from git import Repo
from .common import OutputDataSimple, OutputDataItem, OutputResult, DataType


def get_now():
    utc = pytz.timezone('Asia/Shanghai')
    t = datetime.datetime.now(tz=utc).strftime('%Y-%m-%d %H:%M:%S')
    return t


def determine_lang(file_name, value, app_type):
    lang = langid.classify(value)
    lang_type = lang[0]
    if (lang_type == 'zh' and app_type == 'Java'):
        lang_type = 'zh_CN'
    end = file_name.find('.properties')
    start = end - len(lang_type)
    type_from_file_name = file_name[start:end]
    if (lang_type == type_from_file_name):
        return (True, lang_type)
    else:
        return (False, lang_type)


def out_template_file(file_name, error):
    data = OutputDataSimple(type=DataType.File.value, file_path=file_name)
    result = OutputResult(
        No=error['no'],
        Level=error['level'],
        Scope=error['scope'],
        Name=error['name'],
        Data=data.json,
        Comment=error['comment'])
    return result


def out_template_item(file_name, error, key, value, code):
    data = OutputDataSimple(type=DataType.File.value, file_path=file_name)
    data = OutputDataItem(
        type=DataType.Item.value,
        file_path=file_name,
        key_name=key,
        key_value=value,
        lang_code=code,
        info='')
    result = OutputResult(
        No=error['no'],
        Level=error['level'],
        Scope=error['scope'],
        Name=error['name'],
        Data=data.json,
        Comment=error['comment'])
    return result




__author__ = 'painterg'
__email__ = '22396997@qq.com'

import json
import os
import pandas as pd
from .constants import OUTPUT_HEAD, ERROR_1001, ERROR_1002, ERROR_2001
from .common import OutputHead
from .utils import out_template_file, out_template_item, determine_lang, get_now


class BaseCheck:

    count_status = []
    lang_status = []
    check_1001 = True
    check_1002 = True
    check_2001 = True

    """ 抽取文件数据抽取成字典格式：把每份文件的内容抽取成字典表 """

    def extract_dict(self, f):
        pass

    """ 数据内容会被整理成dataFrame格式进行后续的检查工作 """

    def output_result(self, df):
        pass

    """ check errors about the type of 1001 and 1002"""

    def check_file(self, app_type, file_name, value):
        return None

    """ check errors about the type of 1003"""

    def check_item(self, app_type, file_name, value):
        return None

    """
        path：i18n文件路径，不允许有子目录
        regex：获取i18n文件的正则表达式
    """

    def check(self, app_type, path, regex):
        df = self.load_file(path, regex)
        data = []
        ''' 获取df中每一行的数据，name为待翻译的key，value为行数据 '''
        for row in df.iterrows():
            key = row[0]
            value = row[1]
            ''' item为每一行的列数据，item[0]为列头，item[1]为列值 '''
            for item in value.items():
                data1 = self.check_file(app_type, item[0], item[1])
                data2 = self.check_item(app_type, item[0], key, item[1])
                if data1 is not None:
                    data.append(data1)
                if data2 is not None:
                    data.append(data2)
        self.output_result(data)

    def load_file(self, path, regex):
        data = {}
        for file in path.glob(regex):
            assert file.is_file(), 'the directory contain sub directory'
            data[file.name] = self.convert_file(file)
        return pd.DataFrame(data)

    # 读取属性文件
    def convert_file(self, file):
        with file.open('r') as f:
            properties = self.extract_dict(f)
            return properties

    # 把df转化成json文件，并保留中文显示
    def to_json(self, df):
        return df.to_json(force_ascii=False)


class CheckJava(BaseCheck):

    def extract_dict(self, f):
        properties = {}
        for line in f:
            if line.find('=') > 0:
                strs = line.replace('\n', '').split('=')
                properties[strs[0]] = strs[1]
        return properties

    def check_file(self, app_type, file_name, value):
        result = None
        if self.check_file and len(value) == 0 and self.count_status.count(file_name) == 0:
            self.count_status.append(file_name)
            result = out_template_file(file_name, ERROR_1001)
        elif self.check_1002 and len(value) > 0  \
                and not determine_lang(file_name, value, app_type)[0]  \
                and self.lang_status.count(file_name) == 0:

            self.lang_status.append(file_name)
            result = out_template_file(file_name, ERROR_1002)
        return result.json if result is not None else None

    def check_item(self, app_type, file_name, key, value):
        result = None
        if self.check_2001 and len(value) > 0:
            lang_type = determine_lang(file_name, value, app_type)
            if not lang_type[0]:
                result = out_template_item(file_name, ERROR_2001, key, value, lang_type[1])
        return result.json if result is not None else None

    def output_result(self, data):
        app_path = os.getcwd()
        output_head = OutputHead(
            tools=OUTPUT_HEAD['tools'],
            version=OUTPUT_HEAD['version'],
            app_type=OUTPUT_HEAD['app_type'],
            app_path=app_path,
            datetime=get_now(),
            result=data)
        with open(app_path + '/' + 'result.json', mode='w') as f:
            json.dump(output_head.json, f, ensure_ascii=False, indent=3)
            print('check is done...')


class CheckVue(BaseCheck):
    pass


class CheckFlask(BaseCheck):
    pass



"""Main module."""


from .aticheck import CheckJava
from .aticheck import CheckVue
from .aticheck import CheckFlask


class Ati18n:

    def __init__(self, app_type, path):
        self.app_type = app_type
        self.path = path

    def start(self, config_path):
        print(config_path)
        assert self.path.is_dir(), 'argument path is not directory: %s' % self.path
        if (self.app_type.lower() == 'java'.lower()):
            check_obj = CheckJava()
            regex = '*.properties'
        elif (self.app_type.lower() == 'vue'.lower()):
            check_obj = CheckVue()
            regex = ''
        elif (self.app_type.lower() == 'flask'.lower()):
            check_obj = CheckFlask
            regex = '' 
        check_obj.check(self.app_type, self.path, regex)



__author__ = 'painterg'
__email__ = '22396997@qq.com'

import configparser, os, shutil
from git import Repo


class ConfigUtils:

    def __init__(self, app_type, config_path):
        self.app_type = app_type
        config = configparser.ConfigParser()
        config.read(config_path)
        self.config = config

    def get_url(self):
        return self.config[self.app_type]['repository.url']

    def get_temp_storage_prefix(self):
        return self.config[self.app_type]['repository.temp.storage']

    def get_git_name(self):
        dir_suffix = self.get_url().split('/')[-1]
        end = dir_suffix.find('.git')
        return dir_suffix[0:end]

    def get_clone_to_path(self):
        temp_prefix = self.get_temp_storage_prefix()
        dir_name = self.get_git_name()
        if temp_prefix[-1] == '/':
            return temp_prefix + dir_name
        else:
            return temp_prefix + '/' + dir_name

    def clone_repository(self):
        git_url = self.get_url()
        dir = self.get_clone_to_path()
        print(dir)
        if os.path.exists(dir):
            shutil.rmtree(dir)
        repo = Repo.clone_from(url=git_url,to_path=dir)
